@@ -35,6 +35,8 @@ val extend_env : t -> t -> t
 
 val add : t -> var:Var.t -> value:string -> t
 
+val mem : t -> var:Var.t -> bool
+
 val remove : t -> var:Var.t -> t
 
 val diff : t -> t -> t
@@ -48,7 +50,3 @@ val of_string_map : string String.Map.t -> t
 val to_map : t -> string Map.t
 
 val iter : t -> f:(string -> string -> unit) -> unit
-
-val cons_path : t -> dir:Path.t -> t
-
-val path : t -> Path.t list
