@@ -2,6 +2,10 @@ include Stdlib.Filename
 
 type t = string
 
+module Extension = struct
+  type nonrec t = t
+end
+
 let split_extension fn =
   let ext = extension fn in
   (String.sub fn ~pos:0 ~len:(String.length fn - String.length ext), ext)
@@ -21,3 +25,10 @@ let analyze_program_name fn =
   else if String.contains fn '/' || (Stdlib.Sys.win32 && String.contains fn '\\')
   then Relative_to_current_dir
   else In_path
+
+let compare = String.compare
+
+let equal = String.equal
+
+module Set = String.Set
+module Map = String.Map
