@@ -17,3 +17,7 @@ val of_string : string -> t option
 val to_string : t -> string
 
 module Infix : Comparator.OPS with type t = t
+
+val shift_left : t -> t -> t
+
+val shift_right : t -> t -> t
