@@ -1,7 +1,5 @@
 (** A type of values with an associative operation and an identity element, for
-    example, integers with addition and zero.
-
-    This module type is accessible as [Stdune.Monoid.Basic] outside of [Stdune]. *)
+    example, integers with addition and zero. *)
 module type Basic = sig
   type t
 
@@ -19,9 +17,7 @@ end
 
 (** This module type extends the basic definition of a monoid by adding a
     convenient operator synonym [( @ ) = combine], as well as derived functions
-    [reduce] and [map_reduce].
-
-    This module type is accessible as just [Stdune.Monoid] outside of [Stdune]. *)
+    [reduce] and [map_reduce]. *)
 module type S = sig
   include Basic
 
@@ -34,3 +30,22 @@ module type S = sig
 
   val map_reduce : f:('a -> t) -> 'a list -> t
 end
+
+module Commutative = struct
+  (** Like [Basic] but requires [combine] to be commutative.
+
+      The [combine_is_commutative] type is a "proof" of commutativity, so that
+      one can't simply pass any monoid where a commutative monoid is expected. *)
+  module type Basic = sig
+    include Basic
+
+    type combine_is_commutative = unit
+  end
+
+  (** Like [S] but requires [combine] to be commutative. *)
+  module type S = sig
+    include S
+
+    type combine_is_commutative = unit
+  end
+end
