@@ -25,127 +25,6 @@ let explode_path =
       | "." :: xs -> xs
       | xs -> xs
 
-module External : sig
-  include Path_intf.S
-
-  val relative : t -> string -> t
-
-  val mkdir_p : ?perms:int -> t -> unit
-
-  val initial_cwd : t
-
-  val cwd : unit -> t
-
-  val as_local : t -> string
-end = struct
-  module Table = String.Table
-
-  type t = string
-
-  let to_string t = t
-
-  let equal = String.equal
-
-  let hash = String.hash
-
-  let compare = String.compare
-
-  let extend_basename t ~suffix = t ^ suffix
-
-  let of_string t =
-    if Filename.is_relative t then
-      Code_error.raise "Path.External.of_string: relative path given"
-        [ ("t", String t) ];
-    t
-
-  let parse_string_exn ~loc t =
-    if Filename.is_relative t then
-      User_error.raise ~loc [ Pp.textf "path %s is not absolute" t ];
-    t
-
-  let to_dyn t = Dyn.String t
-
-  (* let rec cd_dot_dot t = match Unix.readlink t with | exception _ ->
-     Filename.dirname t | t -> cd_dot_dot t
-
-     let relative initial_t path = let rec loop t components = match components
-     with | [] | ["." | ".."] -> die "invalid filename concatenation: %s / %s"
-     initial_t path | [fn] -> Filename.concat t fn | "." :: rest -> loop t rest
-     | ".." :: rest -> loop (cd_dot_dot t) rest | comp :: rest -> loop
-     (Filename.concat t comp) rest in loop initial_t (explode_path path) *)
-
-  let relative x y =
-    match y with
-    | "." -> x
-    | _ -> Filename.concat x y
-
-  let basename t = Filename.basename t
-
-  let root = of_string "/"
-
-  let is_root = equal root
-
-  let basename_opt = basename_opt ~is_root ~basename
-
-  let parent t = if is_root t then None else Some (Filename.dirname t)
-
-  let parent_exn t =
-    match parent t with
-    | None ->
-      Code_error.raise "Path.External.parent_exn called on a root path" []
-    | Some p -> p
-
-  let mkdir_p ?perms path =
-    ignore (Fpath.mkdir_p ?perms path : Fpath.mkdir_p_result)
-
-  let unlink_no_err t = Fpath.unlink_no_err t
-
-  let extension t = Filename.extension t
-
-  let split_extension t =
-    let s, ext = Filename.split_extension t in
-    (s, ext)
-
-  let set_extension t ~ext =
-    let base, _ = split_extension t in
-    base ^ ext
-
-  let map_extension t ~f =
-    let base, ext = split_extension t in
-    base ^ f ext
-
-  let cwd () = Sys.getcwd ()
-
-  let initial_cwd = Fpath.initial_cwd
-
-  let as_local t =
-    let s = t in
-    "." ^ s
-
-  include (
-    Comparator.Operators (struct
-      type nonrec t = t
-
-      let compare = compare
-    end) :
-      Comparator.OPS with type t := t)
-
-  let to_string_maybe_quoted t = String.maybe_quoted (to_string t)
-
-  let is_descendant b ~of_:a =
-    if is_root a then true
-    else String.is_prefix ~prefix:(to_string a ^ "/") (to_string b)
-
-  module Map = String.Map
-
-  module Set = struct
-    include String.Set
-
-    let of_listing ~dir ~filenames =
-      of_list_map filenames ~f:(fun f -> relative dir f)
-  end
-end
-
 module Unspecified = Path_intf.Unspecified
 
 module Local_gen : sig
@@ -487,12 +366,146 @@ end = struct
   let basename_opt = basename_opt ~is_root ~basename
 end
 
+module External : sig
+  include Path_intf.S
+
+  val relative : t -> string -> t
+
+  val mkdir_p : ?perms:int -> t -> unit
+
+  val initial_cwd : t
+
+  val cwd : unit -> t
+
+  val as_local : t -> string
+
+  val append_local : t -> Local.t -> t
+
+  val of_filename_relative_to_initial_cwd : string -> t
+end = struct
+  module Table = String.Table
+
+  type t = string
+
+  let to_string t = t
+
+  let equal = String.equal
+
+  let hash = String.hash
+
+  let compare = String.compare
+
+  let extend_basename t ~suffix = t ^ suffix
+
+  let of_string t =
+    if Filename.is_relative t then
+      Code_error.raise "Path.External.of_string: relative path given"
+        [ ("t", String t) ];
+    t
+
+  let parse_string_exn ~loc t =
+    if Filename.is_relative t then
+      User_error.raise ~loc [ Pp.textf "path %s is not absolute" t ];
+    t
+
+  let to_dyn t = Dyn.variant "External" [ Dyn.string t ]
+
+  (* let rec cd_dot_dot t = match Unix.readlink t with | exception _ ->
+     Filename.dirname t | t -> cd_dot_dot t
+
+     let relative initial_t path = let rec loop t components = match components
+     with | [] | ["." | ".."] -> die "invalid filename concatenation: %s / %s"
+     initial_t path | [fn] -> Filename.concat t fn | "." :: rest -> loop t rest
+     | ".." :: rest -> loop (cd_dot_dot t) rest | comp :: rest -> loop
+     (Filename.concat t comp) rest in loop initial_t (explode_path path) *)
+
+  let relative x y =
+    match y with
+    | "." -> x
+    | _ -> Filename.concat x y
+
+  let append_local t local = relative t (Local.to_string local)
+
+  let basename t = Filename.basename t
+
+  let root = of_string "/"
+
+  let is_root = equal root
+
+  let basename_opt = basename_opt ~is_root ~basename
+
+  let parent t = if is_root t then None else Some (Filename.dirname t)
+
+  let parent_exn t =
+    match parent t with
+    | None ->
+      Code_error.raise "Path.External.parent_exn called on a root path" []
+    | Some p -> p
+
+  let mkdir_p ?perms path =
+    ignore (Fpath.mkdir_p ?perms path : Fpath.mkdir_p_result)
+
+  let unlink_no_err t = Fpath.unlink_no_err t
+
+  let extension t = Filename.extension t
+
+  let split_extension t =
+    let s, ext = Filename.split_extension t in
+    (s, ext)
+
+  let set_extension t ~ext =
+    let base, _ = split_extension t in
+    base ^ ext
+
+  let map_extension t ~f =
+    let base, ext = split_extension t in
+    base ^ f ext
+
+  let cwd () = Sys.getcwd ()
+
+  let initial_cwd = Fpath.initial_cwd
+
+  let as_local t =
+    let s = t in
+    "." ^ s
+
+  let of_filename_relative_to_initial_cwd fn =
+    if Filename.is_relative fn then relative initial_cwd fn else of_string fn
+
+  include (
+    Comparator.Operators (struct
+      type nonrec t = t
+
+      let compare = compare
+    end) :
+      Comparator.OPS with type t := t)
+
+  let to_string_maybe_quoted t = String.maybe_quoted (to_string t)
+
+  let is_descendant b ~of_:a =
+    if is_root a then true
+    else String.is_prefix ~prefix:(to_string a ^ "/") (to_string b)
+
+  module Map = String.Map
+
+  module Set = struct
+    include String.Set
+
+    let of_listing ~dir ~filenames =
+      of_list_map filenames ~f:(fun f -> relative dir f)
+  end
+end
+
 module Relative_to_source_root = struct
   let mkdir_p ?perms path =
     ignore (Fpath.mkdir_p ?perms (Local.to_string path) : Fpath.mkdir_p_result)
 end
 
-module Source0 = Local
+module Source0 = struct
+  include Local
+
+  let to_dyn s = Dyn.variant "In_source_tree" [ to_dyn s ]
+end
 
 let abs_root, set_root =
   let root_dir = ref None in
@@ -530,7 +543,9 @@ module Outside_build_dir = struct
     | In_source_dir t -> Local.to_string t
     | External t -> External.to_string t
 
-  let to_dyn t = Dyn.String (to_string t)
+  let to_dyn = function
+    | In_source_dir t -> Source0.to_dyn t
+    | External t -> External.to_dyn t
 
   let of_string s =
     if Filename.is_relative s then In_source_dir (Local.of_string s)
@@ -540,6 +555,16 @@ module Outside_build_dir = struct
     | In_source_dir t -> Relative_to_source_root.mkdir_p ?perms t
     | External t -> External.mkdir_p ?perms t
 
+  let relative t s =
+    match t with
+    | In_source_dir t -> In_source_dir (Local.relative t s)
+    | External t -> External (External.relative t s)
+
+  let extend_basename t ~suffix =
+    match t with
+    | In_source_dir t -> In_source_dir (Local.extend_basename t ~suffix)
+    | External t -> External (External.extend_basename t ~suffix)
+
   let append_local x y =
     match x with
     | In_source_dir x -> In_source_dir (Local.append x y)
@@ -699,6 +724,8 @@ module Build = struct
   let lstat t = Unix.lstat (to_string t)
 
   let unlink_no_err t = Fpath.unlink_no_err (to_string t)
+
+  let to_dyn s = Dyn.variant "In_build_dir" [ to_dyn s ]
 end
 
 module T : sig
@@ -749,12 +776,10 @@ end = struct
 
   let external_ e = External e
 
-  let to_dyn t =
-    let open Dyn in
-    match t with
-    | In_build_dir s -> Variant ("In_build_dir", [ Local.to_dyn s ])
-    | In_source_tree s -> Variant ("In_source_tree", [ Local.to_dyn s ])
-    | External s -> Variant ("External", [ External.to_dyn s ])
+  let to_dyn = function
+    | In_build_dir s -> Build.to_dyn s
+    | In_source_tree s -> Source0.to_dyn s
+    | External s -> External.to_dyn s
 end
 
 include T
@@ -823,17 +848,8 @@ let parse_string_exn ~loc s =
 
 let of_string s = parse_string_exn ~loc:Loc0.none s
 
-let to_dyn =
-  let open Dyn in
-  function
-  | In_build_dir s -> variant "In_build_dir" [ Local.to_dyn s ]
-  | In_source_tree s -> variant "In_source_tree" [ Local.to_dyn s ]
-  | External s -> variant "External" [ External.to_dyn s ]
-
 let of_filename_relative_to_initial_cwd fn =
-  external_
-    (if Filename.is_relative fn then External.relative External.initial_cwd fn
-    else External.of_string fn)
+  external_ (External.of_filename_relative_to_initial_cwd fn)
 
 let to_absolute_filename t =
   Outside_build_dir.to_absolute_filename (local_or_external t)
@@ -975,8 +991,6 @@ let extract_build_context = function
   | In_build_dir p when Local.is_root p -> None
   | In_build_dir t -> Build.extract_build_context t
 
-let extract_build_dir_first_component = extract_build_context
-
 let extract_build_context_exn t =
   match extract_build_context t with
   | Some t -> t
@@ -1119,16 +1133,6 @@ let extend_basename t ~suffix =
   | In_build_dir t -> in_build_dir (Local.extend_basename t ~suffix)
   | External t -> external_ (External.extend_basename t ~suffix)
 
-let insert_after_build_dir_exn =
-  let error a b =
-    Code_error.raise "Path.insert_after_build_dir_exn"
-      [ ("path", to_dyn a); ("insert", String b) ]
-  in
-  fun a b ->
-    match a with
-    | In_build_dir a -> in_build_dir (Local.append (Local.of_string b) a)
-    | In_source_tree _ | External _ -> error a b
-
 let clear_dir dir = Fpath.clear_dir (to_string dir)
 
 let rm_rf ?(allow_external = false) t =
@@ -1265,6 +1269,18 @@ let chmod t ~mode = Unix.chmod (to_string t) mode
 let follow_symlink path =
   Fpath.follow_symlink (to_string path) |> Result.map ~f:of_string
 
+let drop_prefix path ~prefix =
+  String.drop_prefix (to_string path) ~prefix:(to_string prefix)
+  |> Option.map ~f:(fun p ->
+         String.drop_prefix_if_exists ~prefix:"/" p |> Local.of_string)
+
+let drop_prefix_exn t ~prefix =
+  match drop_prefix t ~prefix with
+  | None ->
+    Code_error.raise "Path.drop_prefix_exn"
+      [ ("t", to_dyn t); ("prefix", to_dyn prefix) ]
+  | Some p -> p
+
 module Expert = struct
   let drop_absolute_prefix ~prefix p =
     match
